/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2012 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */
package org.ow2.weblab.petals;

import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessagingException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;
import javax.xml.transform.dom.DOMResult;

import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.message.ExchangeImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

public abstract class AbstractMethod<Args, Ret> {
	protected ComponentContext ctx;
	protected Logger logger;
	protected QName serviceName;
	private JAXBContext jc;

	public AbstractMethod(final QName serviceName, final ComponentContext ctx, final Logger logger) {
		super();
		this.serviceName = serviceName;
		this.ctx = ctx;
		this.jc = initJC();
		this.logger = logger;
	}

	protected Ret call(final Args args) throws MessagingException, JAXBException {
		ExchangeImpl exchange = new ExchangeImpl(ctx.getDeliveryChannel().createExchangeFactory().createExchange(MEPConstants.IN_OUT_PATTERN.value()));
		exchange.setOperation(getOperation());
		exchange.setService(serviceName);
		exchange.setInMessageContent(generateInMessageContent(args));
		logger.info("Calling: " + serviceName);
		this.ctx.getDeliveryChannel().sendSync(exchange.getMessageExchange());
		logger.info("Get reply from: " + serviceName);
		if (exchange.isErrorStatus()) {
			logger.info("Error: " + exchange.getError());
		}
		return generateOutRet(exchange.getOutMessageContentAsDocument());

	}

	private Document generateInMessageContent(final Args args) throws JAXBException {
		Marshaller marshaller;
		try {
			marshaller = this.jc.createMarshaller();
		} catch (JAXBException e) {
			throw new RuntimeException("GRAVE: Error during marshaller instanciation.", e);
		}
		JAXBElement<Args> jaxbElement = new JAXBElement<Args>(getParameterArgsQName(), getParameterArgsClass(), args);
		DOMResult domResult = new DOMResult();
		marshaller.marshal(jaxbElement, domResult);
		return (Document) domResult.getNode();
	}

	private Ret generateOutRet(final Node node) throws JAXBException {
		Unmarshaller unmarshaller = null;
		try {
			unmarshaller = this.jc.createUnmarshaller();
		} catch (JAXBException e) {
			throw new RuntimeException("GRAVE: Error during marshaller instanciation.", e);
		}
		Ret ret = null;
		try {
			ret = unmarshaller.unmarshal(node, getParameterRetClass()).getValue();
		} catch (JAXBException e) {
			throw e;
			//TODO manage exception handling and mapping to real WebLab exceptions
//			generateException(node);
		}
		return ret;
	}

	
//	/**
//	 * 
//	 * @param node
//	 */
//	private void generateException(final Node node) {
//		logger.info("EXCEPTION: ");
//		XMLUtils.outputDOM(node, System.out);
//
//	}

	protected abstract QName getOperation();

	protected abstract QName getParameterArgsQName();

	protected abstract QName getParameterRetQName();

	protected abstract Class<Args> getParameterArgsClass();

	protected abstract Class<Ret> getParameterRetClass();

	protected JAXBContext initJC() {
		try {
			return JAXBContext.newInstance(getParameterArgsClass(), getParameterRetClass());
		} catch (JAXBException e) {
			throw new RuntimeException("GRAVE: Error during marshaller instanciation.", e);
		}
	}

}
