/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2012 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */
package org.ow2.weblab.petals;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessagingException;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;

import org.ow2.weblab.core.services.AccessDeniedException;
import org.ow2.weblab.core.services.Configurable;
import org.ow2.weblab.core.services.ContentNotAvailableException;
import org.ow2.weblab.core.services.InsufficientResourcesException;
import org.ow2.weblab.core.services.InvalidParameterException;
import org.ow2.weblab.core.services.UnexpectedException;
import org.ow2.weblab.core.services.UnsupportedRequestException;
import org.ow2.weblab.core.services.configurable.ConfigureArgs;
import org.ow2.weblab.core.services.configurable.ConfigureReturn;
import org.ow2.weblab.core.services.configurable.ResetConfigurationArgs;
import org.ow2.weblab.core.services.configurable.ResetConfigurationReturn;

public class JbiConfigure implements Configurable {
	private static final String CONFIGURABLE_NS = "http://weblab.ow2.org/core/1.2/services/configurable";
	protected AbstractMethod<ConfigureArgs, ConfigureReturn> innerJbiConfigure;
	protected AbstractMethod<ResetConfigurationArgs, ResetConfigurationReturn> innerJbiResetConfigure;
	protected Logger logger;

	public JbiConfigure(final QName serviceName, final ComponentContext ctx, final Logger logger) throws JAXBException {
		this.logger = logger;
		innerJbiConfigure = new AbstractMethod<ConfigureArgs, ConfigureReturn>(serviceName, ctx, logger) {

			@Override
			protected QName getParameterRetQName() {
				return new QName(CONFIGURABLE_NS, "configureReturn");
			}

			@Override
			protected Class<ConfigureReturn> getParameterRetClass() {
				return ConfigureReturn.class;
			}

			@Override
			protected QName getParameterArgsQName() {
				return new QName(CONFIGURABLE_NS, "configureArgs");
			}

			@Override
			protected Class<ConfigureArgs> getParameterArgsClass() {
				return ConfigureArgs.class;
			}

			@Override
			protected QName getOperation() {
				return new QName("configure");
			}

		};

		innerJbiResetConfigure = new AbstractMethod<ResetConfigurationArgs, ResetConfigurationReturn>(serviceName, ctx, logger) {

			@Override
			protected QName getParameterRetQName() {
				return new QName(CONFIGURABLE_NS, "resetConfigurationReturn");
			}

			@Override
			protected Class<ResetConfigurationReturn> getParameterRetClass() {
				return ResetConfigurationReturn.class;
			}

			@Override
			protected QName getParameterArgsQName() {
				return new QName(CONFIGURABLE_NS, "resetConfigurationArgs");
			}

			@Override
			protected Class<ResetConfigurationArgs> getParameterArgsClass() {
				return ResetConfigurationArgs.class;
			}

			@Override
			protected QName getOperation() {
				return new QName("resetConfiguration");
			}
		};

	}

	@Override
	public ConfigureReturn configure(final ConfigureArgs args) throws AccessDeniedException, ContentNotAvailableException, InsufficientResourcesException,
			InvalidParameterException, UnexpectedException, UnsupportedRequestException {
		try {
			return innerJbiConfigure.call(args);
		} catch (MessagingException e) {
			logger.log(Level.WARNING, "Error during call", e);

			throw new UnexpectedException("Error during call", e.getMessage(), e);

		} catch (JAXBException e) {
			// TODO manage exception WL.
			logger.log(Level.WARNING, "Jaxb error, unable to instanciate the response writer.", e);

			throw new UnexpectedException("Jaxb error, unable to instanciate the response writer.", e.getMessage(), e);
		}

	}

	@Override
	public ResetConfigurationReturn resetConfiguration(final ResetConfigurationArgs args) throws AccessDeniedException, ContentNotAvailableException,
			InsufficientResourcesException, InvalidParameterException, UnexpectedException, UnsupportedRequestException {
		try {
			return innerJbiResetConfigure.call(args);
		} catch (MessagingException e) {
			logger.log(Level.WARNING, "Error during call", e);

			throw new UnexpectedException("Error during call", e.getMessage(), e);

		} catch (JAXBException e) {
			// TODO manage exception WL.
			logger.log(Level.WARNING, "Jaxb error, unable to instanciate the response writer.", e);

			throw new UnexpectedException("Jaxb error, unable to instanciate the response writer.", e.getMessage(), e);
		}

	}

}
