/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2012 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */
package org.ow2.weblab.petals;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessagingException;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;

import org.ow2.weblab.core.services.AccessDeniedException;
import org.ow2.weblab.core.services.ContentNotAvailableException;
import org.ow2.weblab.core.services.InsufficientResourcesException;
import org.ow2.weblab.core.services.InvalidParameterException;
import org.ow2.weblab.core.services.ReportProvider;
import org.ow2.weblab.core.services.UnexpectedException;
import org.ow2.weblab.core.services.UnsupportedRequestException;
import org.ow2.weblab.core.services.reportprovider.AddInformationArgs;
import org.ow2.weblab.core.services.reportprovider.AddInformationReturn;
import org.ow2.weblab.core.services.reportprovider.BuildReportArgs;
import org.ow2.weblab.core.services.reportprovider.BuildReportReturn;

public class JbiReportProvider implements ReportProvider {
	private static final String REPORTPROVIDER_NS = "http://weblab.ow2.org/core/1.2/services/reportprovider";
	protected AbstractMethod<AddInformationArgs, AddInformationReturn> innerJbiAddInformation;
	protected AbstractMethod<BuildReportArgs, BuildReportReturn> innerJbiBuildReport;
	protected Logger logger;

	public JbiReportProvider(final QName serviceName, final ComponentContext ctx, final Logger logger) throws JAXBException {
		this.logger = logger;
		innerJbiAddInformation = new AbstractMethod<AddInformationArgs, AddInformationReturn>(serviceName, ctx, logger) {

			@Override
			protected QName getParameterRetQName() {
				return new QName(REPORTPROVIDER_NS, "addInformationReturn");
			}

			@Override
			protected Class<AddInformationReturn> getParameterRetClass() {
				return AddInformationReturn.class;
			}

			@Override
			protected QName getParameterArgsQName() {
				return new QName(REPORTPROVIDER_NS, "addInformationArgs");
			}

			@Override
			protected Class<AddInformationArgs> getParameterArgsClass() {
				return AddInformationArgs.class;
			}

			@Override
			protected QName getOperation() {
				return new QName("addInformation");
			}

		};

		innerJbiBuildReport = new AbstractMethod<BuildReportArgs, BuildReportReturn>(serviceName, ctx, logger) {

			@Override
			protected QName getParameterRetQName() {
				return new QName(REPORTPROVIDER_NS, "buildReportReturn");
			}

			@Override
			protected Class<BuildReportReturn> getParameterRetClass() {
				return BuildReportReturn.class;
			}

			@Override
			protected QName getParameterArgsQName() {
				return new QName(REPORTPROVIDER_NS, "buildReportArgs");
			}

			@Override
			protected Class<BuildReportArgs> getParameterArgsClass() {
				return BuildReportArgs.class;
			}

			@Override
			protected QName getOperation() {
				return new QName("buildReport");
			}
		};

	}

	@Override
	public AddInformationReturn addInformation(final AddInformationArgs args) throws AccessDeniedException, ContentNotAvailableException,
			InsufficientResourcesException, InvalidParameterException, UnexpectedException, UnsupportedRequestException {
		try {
			return innerJbiAddInformation.call(args);
		} catch (MessagingException e) {
			logger.log(Level.WARNING, "Error during call", e);

			throw new UnexpectedException("Error during call", e.getMessage(), e);

		} catch (JAXBException e) {
			// TODO manage exception WL.
			logger.log(Level.WARNING, "Jaxb error, unable to instanciate the response writer.", e);

			throw new UnexpectedException("Jaxb error, unable to instanciate the response writer.", e.getMessage(), e);
		}

	}

	@Override
	public BuildReportReturn buildReport(final BuildReportArgs args) throws AccessDeniedException, ContentNotAvailableException, InsufficientResourcesException,
			InvalidParameterException, UnexpectedException, UnsupportedRequestException {
		try {
			return innerJbiBuildReport.call(args);
		} catch (MessagingException e) {
			logger.log(Level.WARNING, "Error during call", e);

			throw new UnexpectedException("Error during call", e.getMessage(), e);

		} catch (JAXBException e) {
			// TODO manage exception WL.
			logger.log(Level.WARNING, "Jaxb error, unable to instanciate the response writer.", e);

			throw new UnexpectedException("Jaxb error, unable to instanciate the response writer.", e.getMessage(), e);
		}

	}

}
