/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2012 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */
package org.ow2.weblab.petals;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessagingException;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;

import org.ow2.weblab.core.services.AccessDeniedException;
import org.ow2.weblab.core.services.ContentNotAvailableException;
import org.ow2.weblab.core.services.InsufficientResourcesException;
import org.ow2.weblab.core.services.InvalidParameterException;
import org.ow2.weblab.core.services.ResourceContainer;
import org.ow2.weblab.core.services.UnexpectedException;
import org.ow2.weblab.core.services.UnsupportedRequestException;
import org.ow2.weblab.core.services.resourcecontainer.LoadResourceArgs;
import org.ow2.weblab.core.services.resourcecontainer.LoadResourceReturn;
import org.ow2.weblab.core.services.resourcecontainer.SaveResourceArgs;
import org.ow2.weblab.core.services.resourcecontainer.SaveResourceReturn;

public class JbiResourceContainer implements ResourceContainer {
	private static final String RESOURCECONTAINER_NS = "http://weblab.ow2.org/core/1.2/services/resourcecontainer";
	protected AbstractMethod<SaveResourceArgs, SaveResourceReturn> innerJbiSaveResource;
	protected AbstractMethod<LoadResourceArgs, LoadResourceReturn> innerJbiLoadResource;
	protected Logger logger;

	public JbiResourceContainer(final QName serviceName, final ComponentContext ctx, final Logger logger) throws JAXBException {
		this.logger = logger;
		innerJbiSaveResource = new AbstractMethod<SaveResourceArgs, SaveResourceReturn>(serviceName, ctx, logger) {

			@Override
			protected QName getParameterRetQName() {
				return new QName(RESOURCECONTAINER_NS, "saveResourceReturn");
			}

			@Override
			protected Class<SaveResourceReturn> getParameterRetClass() {
				return SaveResourceReturn.class;
			}

			@Override
			protected QName getParameterArgsQName() {
				return new QName(RESOURCECONTAINER_NS, "saveResourceArgs");
			}

			@Override
			protected Class<SaveResourceArgs> getParameterArgsClass() {
				return SaveResourceArgs.class;
			}

			@Override
			protected QName getOperation() {
				return new QName("saveResource");
			}

		};

		innerJbiLoadResource = new AbstractMethod<LoadResourceArgs, LoadResourceReturn>(serviceName, ctx, logger) {

			@Override
			protected QName getParameterRetQName() {
				return new QName(RESOURCECONTAINER_NS, "loadResourceReturn");
			}

			@Override
			protected Class<LoadResourceReturn> getParameterRetClass() {
				return LoadResourceReturn.class;
			}

			@Override
			protected QName getParameterArgsQName() {
				return new QName(RESOURCECONTAINER_NS, "loadResourceArgs");
			}

			@Override
			protected Class<LoadResourceArgs> getParameterArgsClass() {
				return LoadResourceArgs.class;
			}

			@Override
			protected QName getOperation() {
				return new QName("loadResource");
			}
		};

	}

	@Override
	public SaveResourceReturn saveResource(final SaveResourceArgs args) throws AccessDeniedException, ContentNotAvailableException, InsufficientResourcesException,
			InvalidParameterException, UnexpectedException, UnsupportedRequestException {
		try {
			return innerJbiSaveResource.call(args);
		} catch (MessagingException e) {
			logger.log(Level.WARNING, "Error during call", e);

			throw new UnexpectedException("Error during call", e.getMessage(), e);

		} catch (JAXBException e) {
			// TODO manage exception WL.
			logger.log(Level.WARNING, "Jaxb error, unable to instanciate the response writer.", e);

			throw new UnexpectedException("Jaxb error, unable to instanciate the response writer.", e.getMessage(), e);
		}

	}

	@Override
	public LoadResourceReturn loadResource(final LoadResourceArgs args) throws AccessDeniedException, ContentNotAvailableException, InsufficientResourcesException,
			InvalidParameterException, UnexpectedException, UnsupportedRequestException {
		try {
			return innerJbiLoadResource.call(args);
		} catch (MessagingException e) {
			logger.log(Level.WARNING, "Error during call", e);

			throw new UnexpectedException("Error during call", e.getMessage(), e);

		} catch (JAXBException e) {
			// TODO manage exception WL.
			logger.log(Level.WARNING, "Jaxb error, unable to instanciate the response writer.", e);

			throw new UnexpectedException("Jaxb error, unable to instanciate the response writer.", e.getMessage(), e);
		}

	}

}
